//--------------------------------------------------------------
// File     : stm32_ub_mod_global.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_MODGLOBAL_H
#define __STM32F4_UB_MODGLOBAL_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"


//--------------------------------------------------------------
// hier einstellen ob die Ausgabe per
// DAC oder PWM erfolgen soll
// (nur eines von beiden aktivieren)
//--------------------------------------------------------------
#define   UB_MOD_USE_DAC    1
//#define   UB_MOD_USE_PWM    2



//--------------------------------------------------------------
// hier einstellen welche Samplerate benutzt werden soll
// HI = 40kHz (fuer MOD-Files vom TYP : MOD)
// LO = 20kHz (fuer MOD-Files vom TYP : MOD und S3M)
//--------------------------------------------------------------
//#define  UB_MOD_SAMPLERATE_LO   1
#define  UB_MOD_SAMPLERATE_HI   2



//--------------------------------------------------------------
// Globale Defines
//--------------------------------------------------------------
#define  ROWS                 64
#define  CHANNELS             14
#define  INSTRUMENTS          99
#define  SOUNDBUFFERSIZE     512


//--------------------------------------------------------------
// Clock Defines
//--------------------------------------------------------------
#define SYSCLK        168000000L
#ifdef UB_MOD_USE_DAC
  #define BITDEPTH         12
#endif
#ifdef UB_MOD_USE_PWM
  #define BITDEPTH         11
#endif
#define DIVIDER            10


#ifdef UB_MOD_SAMPLERATE_LO
  #define SAMPLERATE (SYSCLK / 2 / 1024 / 4)   // Samplerate = 20kHz
#endif
#ifdef UB_MOD_SAMPLERATE_HI
  #define SAMPLERATE (SYSCLK / 2 / 1024 / 2)   // Samplerate = 40kHz
#endif


#define AMIGA_MOD (7093789 / 2 / SAMPLERATE << DIVIDER)
#define AMIGA_S3M (14317056 / SAMPLERATE << DIVIDER)


//--------------------------------------------------------------
#define min(X,Y) ((X) < (Y) ? (X) : (Y))



//--------------------------------------------------------------
// Struktur vom Mixer
//--------------------------------------------------------------
typedef struct {
  uint32_t sampleBegin[INSTRUMENTS];
  uint32_t sampleEnd[INSTRUMENTS];
  uint32_t sampleLoopBegin[INSTRUMENTS];
  uint16_t sampleLoopLength[INSTRUMENTS];
  uint32_t sampleLoopEnd[INSTRUMENTS];
  uint8_t channelSampleNumber[CHANNELS];
  uint32_t channelSampleOffset[CHANNELS];
  uint16_t channelFrequency[CHANNELS];
  uint8_t channelVolume[CHANNELS];
  uint8_t channelPanning[CHANNELS];
}mixer_t;
mixer_t Mixer;



//--------------------------------------------------------------
// Struktur vom Sound-Buffer
//--------------------------------------------------------------
typedef struct {
  uint16_t left[SOUNDBUFFERSIZE];
  uint16_t right[SOUNDBUFFERSIZE];
  uint16_t writePos;
  volatile uint16_t readPos;
}soundBuffer_t;
soundBuffer_t SoundBuffer;



//--------------------------------------------------------------
// Struktur von einem MOD-File
//--------------------------------------------------------------
typedef struct UB_MOD_File_t
{
  const uint8_t *table; // Tabelle mit den Daten
  uint32_t size;        // Anzahl der Bytes
}UB_MOD_File;



//--------------------------------------------------------------
// Mod-Files die im Programm benutzt werden
// (muessen als C-File auch in der IDE eingebunden sein)
//--------------------------------------------------------------
extern UB_MOD_File   MOD_File_Aurora;






//--------------------------------------------------------------
#endif // __STM32F4_UB_MODGLOBAL_H
