//--------------------------------------------------------------
// File     : stm32_ub_modul_mod.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_MODUL_MOD_H
#define __STM32F4_UB_MODUL_MOD_H



//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include <stdbool.h> // fuer bool
#include <stdlib.h>  //fuer RAND
#include "stm32_ub_mod_global.h"



//--------------------------------------------------------------
// Defines fuer MOD
//--------------------------------------------------------------
#define  MOD_NAME_LEN              20
#define  MOD_SAMPLES               31
#define  MOD_SAMPLE_NAME_LEN       22
#define  MOD_SAMPLE_OFFSET       1084
#define  MOD_STEREOSEPARATION      32
#define  MOD_NONOTE            0xFFFF



//--------------------------------------------------------------
// Effecte
//--------------------------------------------------------------
#define MOD_ARPEGGIO              0x0
#define MOD_PORTAMENTOUP          0x1
#define MOD_PORTAMENTODOWN        0x2
#define MOD_TONEPORTAMENTO        0x3
#define MOD_VIBRATO               0x4
#define MOD_PORTAMENTOVOLUMESLIDE 0x5
#define MOD_VIBRATOVOLUMESLIDE    0x6
#define MOD_TREMOLO               0x7
#define MOD_SETCHANNELPANNING     0x8
#define MOD_SETSAMPLEOFFSET       0x9
#define MOD_VOLUMESLIDE           0xA
#define MOD_JUMPTOORDER           0xB
#define MOD_SETVOLUME             0xC
#define MOD_BREAKPATTERNTOROW     0xD
#define MOD_SPECIALEFFECT_E       0xE
#define MOD_SETSPEED              0xF


//--------------------------------------------------------------
// Effecte (vom TYP MOD_SPECIALEFFECT_E)
//--------------------------------------------------------------
#define MOD_SETFILTER             0x0
#define MOD_FINEPORTAMENTOUP      0x1
#define MOD_FINEPORTAMENTODOWN    0x2
#define MOD_GLISSANDOCONTROL      0x3
#define MOD_SETVIBRATOWAVEFORM    0x4
#define MOD_SETFINETUNE           0x5
#define MOD_PATTERNLOOP           0x6
#define MOD_SETTREMOLOWAVEFORM    0x7
#define MOD_RETRIGGERNOTE         0x9
#define MOD_FINEVOLUMESLIDEUP     0xA
#define MOD_FINEVOLUMESLIDEDOWN   0xB
#define MOD_NOTECUT               0xC
#define MOD_NOTEDELAY             0xD
#define MOD_PATTERNDELAY          0xE
#define MOD_INVERTLOOP            0xF



//--------------------------------------------------------------
// Struktur vom MOD
//--------------------------------------------------------------
typedef struct {
  uint8_t name[MOD_SAMPLE_NAME_LEN+1];
  uint16_t length;
  int8_t fineTune;
  uint8_t volume;
  uint16_t loopBegin;
  uint16_t loopLength;
}MOD_Sample_t;


typedef struct {
  uint8_t name[MOD_NAME_LEN+1];
  MOD_Sample_t samples[MOD_SAMPLES];
  uint8_t songLength;
  uint8_t numberOfPatterns;
  uint8_t order[128];
  uint8_t numberOfChannels;
  uint8_t id[5];
}mod_t;
mod_t Mod;



//--------------------------------------------------------------
// Struktur vom MOD-Player
//--------------------------------------------------------------
typedef struct {
  uint8_t sampleNumber[ROWS][CHANNELS];
  uint16_t note[ROWS][CHANNELS];
  uint8_t effectNumber[ROWS][CHANNELS];
  uint8_t effectParameter[ROWS][CHANNELS];
}MOD_Pattern_t;


typedef struct {
  MOD_Pattern_t currentPattern;
  uint32_t amiga;
  uint16_t samplesPerTick;
  uint8_t speed;
  uint8_t tick;
  uint8_t row;
  uint8_t lastRow;
  uint8_t orderIndex;
  uint8_t oldOrderIndex;
  uint8_t patternDelay;
  uint8_t patternLoopCount[CHANNELS];
  uint8_t patternLoopRow[CHANNELS];
  uint8_t lastSampleNumber[CHANNELS];
  int8_t volume[CHANNELS];
  uint16_t lastNote[CHANNELS];
  uint16_t amigaPeriod[CHANNELS];
  int16_t lastAmigaPeriod[CHANNELS];
  uint16_t portamentoNote[CHANNELS];
  uint8_t portamentoSpeed[CHANNELS];
  uint8_t waveControl[CHANNELS];
  uint8_t vibratoSpeed[CHANNELS];
  uint8_t vibratoDepth[CHANNELS];
  int8_t vibratoPos[CHANNELS];
  uint8_t tremoloSpeed[CHANNELS];
  uint8_t tremoloDepth[CHANNELS];
  int8_t tremoloPos[CHANNELS];
}mod_player_t;
mod_player_t Mod_player;



//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
uint8_t UB_MOD_Init(UB_MOD_File *modfile);
uint8_t UB_MOD_Play(UB_MOD_File *modfile, uint8_t loop);
void UB_MOD_Mixer(UB_MOD_File *modfile);



//--------------------------------------------------------------
#endif // __STM32F4_UB_MODUL_MOD_H
