//--------------------------------------------------------------
// File     : stm32_ub_modul_s3m.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_MODUL_S3M_H
#define __STM32F4_UB_MODUL_S3M_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include <stdbool.h> // fuer bool
#include <stdlib.h>  //fuer RAND
#include <string.h> // fuer MEMSET
#include "stm32_ub_mod_global.h"



//--------------------------------------------------------------
// Defines fuer S3M
//--------------------------------------------------------------
#define  S3M_NAME_LEN             28
#define  S3M_NONOTE              108
#define  S3M_KEYOFF              109
#define  S3M_NOVOLUME            255



//--------------------------------------------------------------
// Effecte
//--------------------------------------------------------------
#define S3M_SETSPEED                 0x1  // Axx
#define S3M_JUMPTOORDER              0x2  // Bxx
#define S3M_BREAKPATTERNTOROW        0x3  // Cxx
#define S3M_VOLUMESLIDE              0x4  // Dxx
#define S3M_PORTAMENTODOWN           0x5  // Exx
#define S3M_PORTAMENTOUP             0x6  // Fxx
#define S3M_TONEPORTAMENTO           0x7  // Gxx
#define S3M_VIBRATO                  0x8  // Hxy
#define S3M_TREMOR                   0x9  // Ixy
#define S3M_ARPEGGIO                 0xA  // Jxy
#define S3M_VIBRATOVOLUMESLIDE       0xB  // Kxy
#define S3M_PORTAMENTOVOLUMESLIDE    0xC  // Lxy
#define S3M_SETSAMPLEOFFSET          0xF  // Oxy
#define S3M_RETRIGGERNOTEVOLUMESLIDE 0x11 // Qxy
#define S3M_TREMOLO                  0x12 // Rxy
#define S3M_SPECIALEFFECT_13         0x13
#define S3M_SETTEMPO                 0x14 // Txx
#define S3M_FINEVIBRATO              0x15 // Uxy
#define S3M_SETGLOBALVOLUME          0x16 // Vxx


//--------------------------------------------------------------
// Effecte (vom TYP S3M_SPECIALEFFECT_13)
//--------------------------------------------------------------
#define S3M_SETFILTER                0x0
#define S3M_SETGLISSANDOCONTROL      0x1
#define S3M_SETFINETUNE              0x2
#define S3M_SETVIBRATOWAVEFORM       0x3
#define S3M_SETTREMOLOWAVEFORM       0x4
#define S3M_SETCHANNELPANNING        0x8
#define S3M_STEREOCONTROL            0xA
#define S3M_PATTERNLOOP              0xB
#define S3M_NOTECUT                  0xC
#define S3M_NOTEDELAY                0xD
#define S3M_PATTERNDELAY             0xE
#define S3M_FUNKREPEAT               0xF



//--------------------------------------------------------------
// Struktur vom S3M
//--------------------------------------------------------------
typedef struct {
  uint8_t name[S3M_NAME_LEN+1];
  uint16_t sampleParapointer;
  uint16_t length;
  uint16_t loopBegin;
  uint16_t loopEnd;
  uint8_t volume;
  uint16_t middleC;
  bool loop;
}S3M_Instrument_t;


typedef struct {
  uint8_t name[S3M_NAME_LEN+1];
  S3M_Instrument_t instruments[INSTRUMENTS];
  uint16_t songLength;
  uint16_t numberOfInstruments;
  uint16_t numberOfPatterns;
  bool fastVolumeSlides;
  uint8_t globalVolume;
  uint8_t order[256];
  uint8_t numberOfChannels;
  uint8_t channelRemapping[CHANNELS];
  uint16_t instrumentParapointers[INSTRUMENTS];
  uint16_t patternParapointers[256];
}s3m_t;
s3m_t S3m;



//--------------------------------------------------------------
// Struktur vom S3M-Player
//--------------------------------------------------------------
typedef struct{
  uint16_t note[ROWS][CHANNELS];
  uint8_t instrumentNumber[ROWS][CHANNELS];
  uint8_t volume[ROWS][CHANNELS];
  uint8_t effectNumber[ROWS][CHANNELS];
  uint8_t effectParameter[ROWS][CHANNELS];
}S3M_Pattern_t;


typedef struct {
  S3M_Pattern_t currentPattern;
  uint32_t amiga;
  uint16_t samplesPerTick;
  uint8_t speed;
  uint8_t tick;
  uint8_t row;
  uint8_t lastRow;
  uint8_t orderIndex;
  uint8_t oldOrderIndex;
  uint8_t patternDelay;
  uint8_t patternLoopCount[CHANNELS];
  uint8_t patternLoopRow[CHANNELS];
  uint8_t lastInstrumentNumber[CHANNELS];
  int8_t volume[CHANNELS];
  uint16_t lastNote[CHANNELS];
  uint16_t amigaPeriod[CHANNELS];
  int16_t lastAmigaPeriod[CHANNELS];
  uint8_t lastVolumeSlide[CHANNELS];
  uint8_t lastPortamento[CHANNELS];
  uint16_t portamentoNote[CHANNELS];
  uint8_t portamentoSpeed[CHANNELS];
  uint8_t waveControl[CHANNELS];
  uint8_t vibratoSpeed[CHANNELS];
  uint8_t vibratoDepth[CHANNELS];
  int8_t vibratoPos[CHANNELS];
  uint8_t tremoloSpeed[CHANNELS];
  uint8_t tremoloDepth[CHANNELS];
  int8_t tremoloPos[CHANNELS];
  uint8_t tremorOnOff[CHANNELS];
  uint8_t tremorCount[CHANNELS];
  uint8_t retriggerVolumeSlide[CHANNELS];
  uint8_t retriggerSpeed[CHANNELS];
}s3m_player_t;
s3m_player_t S3m_player;



//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
uint8_t UB_S3M_Init(UB_MOD_File *modfile);
uint8_t UB_S3M_Play(UB_MOD_File *modfile, uint8_t loop);
void UB_S3M_Mixer(UB_MOD_File *modfile);



//--------------------------------------------------------------
#endif // __STM32F4_UB_MODUL_S3M_H
