//--------------------------------------------------------------
// File     : stm32_ub_modplayer.c
// Datum    : 05.06.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : TIM, MISC, MODUL_MOD, MODUL_S3M
//            STM32_UB_DAC (falls Output per DAC)
//            STM32_UB_PWM (falls Output per PWM)
//
// Funktion : Abspielen von MOD-Files aus dem Flash
//            Modul-Typ : MOD oder S3M
//
// Hinweise : (DAC oder PWM im File : Global.h einstellen)
//            fuer S3M-Files, Samplerate auf "LO" einstellen
//
// DAC-Out  : PA4 (left), PA5 (right)
// PWM-Out  : PB11 (left), PA1 (right)
// RC-Filter fuer die Ausgnge notwendig
//--------------------------------------------------------------


//--------------------------------------------------------------
// Project ported from "Stellarplayer by Ronen.K (V:0.1)"
// follow this Link : http://mobile4dev.blogspot.co.il/
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_modplayer.h"


//--------------------------------------------------------------
// Globale Variabeln
//--------------------------------------------------------------
MODPLAYER_STATUS_t modplayer_status=MODPLAYER_NO_INIT;



//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_Mod_InitTIM(void);
void P_Mod_InitNVIC(void);


//--------------------------------------------------------------
// init vom Mod-Player
//--------------------------------------------------------------
void UB_Modplayer_Init(void)
{
  if(modplayer_status==MODPLAYER_NO_INIT) {
    // init VAR
    MODPLAYER.loop_flag=0;
    MODPLAYER.mod_filetype=0;

    // Init vom Timer
    P_Mod_InitTIM();
    // Init vom NVIC
    P_Mod_InitNVIC();
    #ifdef UB_MOD_USE_DAC
      // Init vom DAC
      UB_DAC_Init(DUAL_DAC);
    #endif

    #ifdef UB_MOD_USE_PWM
      // init vom PWM
      UB_PWM_TIM2_Init();
    #endif

    modplayer_status=MODPLAYER_INIT_OK;
  }
}


//--------------------------------------------------------------
// spielt ein MOD-File einmal ab (aus dem Flash)
// -> File muss mit &-Operator uebergeben werden
// 
// ret_wert :    MODFILE_OK => Modfile wird abgespielt
//            != MODFILE_OK => Modfile wird nicht abgespielt
//--------------------------------------------------------------
MOD_FILE_t UB_Modplayer_PlaySingle(UB_MOD_File *modfile)
{
  MOD_FILE_t ret_wert=MODFILE_NOTREADY;
  uint8_t check;

  if((modplayer_status==MODPLAYER_INIT_OK) || (modplayer_status==MODPLAYER_STOP)) {

    // nur einmal abspielen
    MODPLAYER.loop_flag=0;
    MODPLAYER.akt_modfile=modfile;
    MODPLAYER.mod_filetype=0;    // Modul=unknown

    check=UB_MOD_Init(MODPLAYER.akt_modfile);
    if(check==0) {
      MODPLAYER.mod_filetype=1;  // Modul=MOD
      modplayer_status=MODPLAYER_PLAY;
      ret_wert=MODFILE_OK;
    }
    else {
      check=UB_S3M_Init(MODPLAYER.akt_modfile);
      if(check==0) {
        MODPLAYER.mod_filetype=2; // Modul=S3M
        modplayer_status=MODPLAYER_PLAY;
        ret_wert=MODFILE_OK;
      }
      else {
        ret_wert=MODFILE_ERROR;
      }
    }
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// spielt ein MOD-File in einer Endloosloop ab (aus dem Flash)
// -> File muss mit &-Operator uebergeben werden
// 
// ret_wert :    MODFILE_OK => Modfile wird abgespielt
//            != MODFILE_OK => Modfile wird nicht abgespielt
//--------------------------------------------------------------
MOD_FILE_t UB_Modplayer_PlayLoop(UB_MOD_File *modfile)
{
  MOD_FILE_t ret_wert=MODFILE_NOTREADY;
  uint8_t check;

  if((modplayer_status==MODPLAYER_INIT_OK) || (modplayer_status==MODPLAYER_STOP)) {

    // in dauerloop abspielen
    MODPLAYER.loop_flag=1;
    MODPLAYER.akt_modfile=modfile;
    MODPLAYER.mod_filetype=0;    // Modul=unknown
	
    check=UB_MOD_Init(MODPLAYER.akt_modfile);
    if(check==0) {
      MODPLAYER.mod_filetype=1;  // Modul=MOD
      modplayer_status=MODPLAYER_PLAY;
      ret_wert=MODFILE_OK;
    }
    else {	
      check=UB_S3M_Init(MODPLAYER.akt_modfile);
      if(check==0) {
        MODPLAYER.mod_filetype=2; // Modul=S3M
        modplayer_status=MODPLAYER_PLAY;
        ret_wert=MODFILE_OK;
      }
      else {
        ret_wert=MODFILE_ERROR;
      }
    }
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// MOD-Play-Funktion
// diese Funktion muss zyklisch aufgerufen werden
// solange das MOD-File abgespielt wird
//
// return_wert :
//     MODPLAYER_PLAY = MOD-File wird noch abgespielt
//  != MODPLAYER_PLAY = MOD-File wird nicht mehr abgespielt
//--------------------------------------------------------------
MODPLAYER_STATUS_t UB_Modplayer_PlayDo(void)
{
  static uint16_t akt_sample=0;
  uint8_t check;

  if(modplayer_status==MODPLAYER_PLAY) {
    if(MODPLAYER.mod_filetype==1) {    
      if(((SoundBuffer.writePos + 1) & (SOUNDBUFFERSIZE - 1)) != SoundBuffer.readPos) {
        if(akt_sample==0) {
          check=UB_MOD_Play(MODPLAYER.akt_modfile,MODPLAYER.loop_flag);
          if(check==1) {
            modplayer_status=MODPLAYER_STOP;
            akt_sample=0;
            return(modplayer_status);
          }
          akt_sample=Mod_player.samplesPerTick;
        }
        UB_MOD_Mixer(MODPLAYER.akt_modfile);
        akt_sample--;
      }
    }
    else if(MODPLAYER.mod_filetype==2) {
      if(((SoundBuffer.writePos + 1) & (SOUNDBUFFERSIZE - 1)) != SoundBuffer.readPos) {
        if(akt_sample==0) {
          check=UB_S3M_Play(MODPLAYER.akt_modfile,MODPLAYER.loop_flag);
          if(check==1) {
            modplayer_status=MODPLAYER_STOP;
            akt_sample=0;
            return(modplayer_status);
          }
          akt_sample=S3m_player.samplesPerTick;
        }
        UB_S3M_Mixer(MODPLAYER.akt_modfile);
        akt_sample--;
      }
    }
  }

  return(modplayer_status);
}


//--------------------------------------------------------------
// Pausen-Funktion
//--------------------------------------------------------------
void UB_Modplayer_Pause(void)
{
  if(modplayer_status==MODPLAYER_PLAY) {
    modplayer_status=MODPLAYER_PAUSE;
  }
}


//--------------------------------------------------------------
// Resume-Funktion
//--------------------------------------------------------------

void UB_Modplayer_Resume(void)
{
  if(modplayer_status==MODPLAYER_PAUSE) {
    modplayer_status=MODPLAYER_PLAY;
  }
}


//--------------------------------------------------------------
// Stop-Funktion
//--------------------------------------------------------------
void UB_Modplayer_Stop(void)
{
  if((modplayer_status==MODPLAYER_PLAY) || (modplayer_status==MODPLAYER_PAUSE)) {
    modplayer_status=MODPLAYER_STOP;
  }
}


//--------------------------------------------------------------
// interne Funktion
// init vom Timer
//--------------------------------------------------------------
void P_Mod_InitTIM(void)
{
  TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;

  // Clock enable
  RCC_APB1PeriphClockCmd(UB_MOD_TIM_CLK, ENABLE);

  // Timer init
  TIM_TimeBaseStructure.TIM_Period =  UB_MOD_TIM_PERIODE;
  TIM_TimeBaseStructure.TIM_Prescaler = UB_MOD_TIM_PRESCALE;
  TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(UB_MOD_TIM, &TIM_TimeBaseStructure);

  // Timer enable
  TIM_ARRPreloadConfig(UB_MOD_TIM, ENABLE);
  TIM_Cmd(UB_MOD_TIM, ENABLE);

}


//--------------------------------------------------------------
// interne Funktion
// init vom NVIC
//--------------------------------------------------------------
void P_Mod_InitNVIC(void)
{
  NVIC_InitTypeDef NVIC_InitStructure;

  //---------------------------------------------
  // init vom Timer Interrupt
  //---------------------------------------------
  TIM_ITConfig(UB_MOD_TIM,TIM_IT_Update,ENABLE);

  // NVIC konfig
  NVIC_InitStructure.NVIC_IRQChannel = UB_MOD_TIM_IRQ;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

}


//--------------------------------------------------------------
// ISR vom Timer
//--------------------------------------------------------------
void UB_MOD_TIM_ISR_HANDLER(void)
{
  uint16_t left,right;

  // es gibt hier nur eine Interrupt Quelle
  TIM_ClearITPendingBit(UB_MOD_TIM, TIM_IT_Update);

  if(SoundBuffer.writePos != SoundBuffer.readPos) {
    left=SoundBuffer.left[SoundBuffer.readPos];
    right=SoundBuffer.right[SoundBuffer.readPos];

    #ifdef UB_MOD_USE_DAC
      UB_DAC_SetDAC1u2(left,right);
    #endif

    #ifdef UB_MOD_USE_PWM
      UB_PWM_TIM2_SetPWM(PWM_LEFT,left);
      UB_PWM_TIM2_SetPWM(PWM_RIGHT,right);
    #endif

    SoundBuffer.readPos++;
    SoundBuffer.readPos &= SOUNDBUFFERSIZE - 1;
  }
}
