//--------------------------------------------------------------
// File     : stm32_ub_modplayer.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_MODPLAYER_H
#define __STM32F4_UB_MODPLAYER_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32_ub_mod_global.h"
#include "stm32_ub_modul_mod.h"
#include "stm32_ub_modul_s3m.h"
#include "stm32f4xx_tim.h"
#include "stm32f4xx_rcc.h"
#include "misc.h"




//--------------------------------------------------------------
#ifdef UB_MOD_USE_DAC
  #include "stm32_ub_dac.h"
#endif

#ifdef UB_MOD_USE_PWM
  #include "stm32_ub_pwm_tim2.h"
#endif




//--------------------------------------------------------------
// Timer fr Mod-Player => TIM7
// Grundfrequenz = 2*APB1 (APB1=42MHz) => TIM_CLK=84MHz
// TIM_Frq = TIM_CLK/(periode+1)/(vorteiler+1)
// TIM_Frq = SAMPLERATE
//--------------------------------------------------------------
#define   UB_MOD_TIM              TIM7
#define   UB_MOD_TIM_CLK          RCC_APB1Periph_TIM7
#define   UB_MOD_TIM_PERIODE      1023
#ifdef UB_MOD_SAMPLERATE_LO
  #define   UB_MOD_TIM_PRESCALE     3
#endif
#ifdef UB_MOD_SAMPLERATE_HI
  #define   UB_MOD_TIM_PRESCALE     1
#endif
#define   UB_MOD_TIM_IRQ          TIM7_IRQn
#define   UB_MOD_TIM_ISR_HANDLER  TIM7_IRQHandler



//--------------------------------------------------------------
// Struktur vom Mod-Player
//--------------------------------------------------------------
typedef struct {
  UB_MOD_File *akt_modfile;
  uint8_t loop_flag;         // 0=single, 1=loop
  uint8_t mod_filetype;      // 0=unknown, 1=MOD, 2=S3M
}MODPLYAER_t;
MODPLYAER_t MODPLAYER;



//--------------------------------------------------------------
typedef enum {
  MODFILE_OK    =0,    // Modfile wird abgespielt
  MODFILE_NOTREADY,    // nicht bereit zum abspielen
  MODFILE_ERROR        // Fehler im Modfile
}MOD_FILE_t;




//--------------------------------------------------------------
typedef enum {
  MODPLAYER_NO_INIT = 0,  // noch nicht initialisiert
  MODPLAYER_INIT_OK,      // init ok
  MODPLAYER_PLAY,         // PLAY
  MODPLAYER_PAUSE,        // PAUSE
  MODPLAYER_STOP          // STOP
}MODPLAYER_STATUS_t;



//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
void UB_Modplayer_Init(void);
MOD_FILE_t UB_Modplayer_PlaySingle(UB_MOD_File *modfile);
MOD_FILE_t UB_Modplayer_PlayLoop(UB_MOD_File *modfile);
MODPLAYER_STATUS_t UB_Modplayer_PlayDo(void);
void UB_Modplayer_Pause(void);
void UB_Modplayer_Resume(void);
void UB_Modplayer_Stop(void);




//--------------------------------------------------------------
#endif // __STM32F4_UB_MODPLAYER_H
